# --------------------------------------------------------
#    mmqgis - QGIS plugins entry class
#
#    begin                : August 5, 2009
#    copyright            : (c) 2009 by Michael Minn
#    email                : See michaelminn.com
#
#   MMQGIS is free software and is offered without guarantee
#   or warranty. You can redistribute it and/or modify it 
#   under the terms of version 2 of the GNU General Public 
#   License (GPL v2) as published by the Free Software 
#   Foundation (www.gnu.org).
# --------------------------------------------------------

from PyQt4.QtCore import *
from PyQt4.QtGui import *
from qgis.core import *

from mmqgis_attribute_export import *
from mmqgis_attribute_join import *
from mmqgis_grid import *
from mmqgis_gridify import *
from mmqgis_label import *
from mmqgis_merge import *
from mmqgis_sort import *
from mmqgis_street_geocode import *
from mmqgis_voronoi import *

# ---------------------------------------------

class mmqgis:
	def __init__(self, iface):
		self.iface = iface

	def initGui(self):
		icon = QIcon(os.path.dirname(__file__) + "/mmqgis_grid.png")
		self.grid_action = QAction(icon, "Create Grid Layer", self.iface.mainWindow())
		QObject.connect(self.grid_action, SIGNAL("activated()"), self.grid)
		self.iface.addPluginToMenu("&mmqgis", self.grid_action)

		icon = QIcon(os.path.dirname(__file__) + "/mmqgis_label.png")
		self.label_action = QAction(icon, "Create Label Layer", self.iface.mainWindow())
		QObject.connect(self.label_action, SIGNAL("activated()"), self.label)
		self.iface.addPluginToMenu("&mmqgis", self.label_action)

		icon = QIcon(os.path.dirname(__file__) + "/mmqgis_gridify.png")
		self.gridify_action = QAction(icon, "Gridify", self.iface.mainWindow())
		QObject.connect(self.gridify_action, SIGNAL("activated()"), self.gridify)
		self.iface.addPluginToMenu("&mmqgis", self.gridify_action)

		icon = QIcon(os.path.dirname(__file__) + "/mmqgis_attribute_export.png")
		self.attribute_export_action = QAction(icon, "Export Attributes to CSV File", self.iface.mainWindow())
		QObject.connect(self.attribute_export_action, SIGNAL("activated()"), self.attribute_export)
		self.iface.addPluginToMenu("&mmqgis", self.attribute_export_action)

		icon = QIcon(os.path.dirname(__file__) + "/mmqgis_attribute_join.png")
		self.attribute_join_action = QAction(icon, "Join Attributes from CSV File", self.iface.mainWindow())
		QObject.connect(self.attribute_join_action, SIGNAL("activated()"), self.attribute_join)
		self.iface.addPluginToMenu("&mmqgis", self.attribute_join_action)

		icon = QIcon(os.path.dirname(__file__) + "/mmqgis_merge.png")
		self.merge_action = QAction(icon, "Merge Layers", self.iface.mainWindow())
		QObject.connect(self.merge_action, SIGNAL("activated()"), self.merge)
		self.iface.addPluginToMenu("&mmqgis", self.merge_action)

		icon = QIcon(os.path.dirname(__file__) + "/mmqgis_sort.png")
		self.sort_action = QAction(icon, "Sort", self.iface.mainWindow())
		QObject.connect(self.sort_action, SIGNAL("activated()"), self.sort)
		self.iface.addPluginToMenu("&mmqgis", self.sort_action)

		icon = QIcon(os.path.dirname(__file__) + "/mmqgis_street_geocode.png")
		self.street_geocode_action = QAction(icon, "Street Address Geocode", self.iface.mainWindow())
		QObject.connect(self.street_geocode_action, SIGNAL("activated()"), self.street_geocode)
		self.iface.addPluginToMenu("&mmqgis", self.street_geocode_action)

		icon = QIcon(os.path.dirname(__file__) + "/mmqgis_voronoi.png")
		self.voronoi_action = QAction(icon, "Voronoi Diagram", self.iface.mainWindow())
		QObject.connect(self.voronoi_action, SIGNAL("activated()"), self.voronoi)
		self.iface.addPluginToMenu("&mmqgis", self.voronoi_action)

	def unload(self):
    		self.iface.removePluginMenu("&mmqgis", self.grid_action)
    		self.iface.removePluginMenu("&mmqgis", self.gridify_action)

	def attribute_export(self):
		dialog = mmqgis_attribute_export(self.iface)
		dialog.exec_()

	def attribute_join(self):
		dialog = mmqgis_attribute_join(self.iface)
		dialog.exec_()

	def grid(self):
		dialog = mmqgis_grid(self.iface)
		dialog.exec_()

	def gridify(self):
		dialog = mmqgis_gridify(self.iface)
		dialog.exec_()

	def label(self):
		dialog = mmqgis_label(self.iface)
		dialog.exec_()

	def merge(self):
		dialog = mmqgis_merge(self.iface)
		dialog.exec_()

	def sort(self):
		#layer = self.iface.activeLayer()
		#if (not layer) or (layer.type() != QgsMapLayer.VectorLayer):
		#	QMessageBox.information(self.iface.mainWindow(), "Sort Layer", \
		#		"Project has no active vector layer to sort")
		#	return

		dialog = mmqgis_sort(self.iface)
		dialog.exec_()

	def street_geocode(self):
		dialog = mmqgis_street_geocode(self.iface)
		dialog.exec_()

	def voronoi(self):
		dialog = mmqgis_voronoi(self.iface)
		dialog.exec_()

