# --------------------------------------------------------
#    mmqgis_attribute_export - Sort shapefile by attribute
#
#    begin                : August 14, 2009
#    copyright            : (c) 2009 by Michael Minn
#    email                : See michaelminn.com
#
#   MMQGIS is free software and is offered without guarantee
#   or warranty. You can redistribute it and/or modify it 
#   under the terms of version 2 of the GNU General Public 
#   License (GPL v2) as published by the Free Software 
#   Foundation (www.gnu.org).
# --------------------------------------------------------

from PyQt4.QtCore import *
from PyQt4.QtGui import *
from qgis.core import *
from mmqgis_attribute_export_form import *

import csv
import os.path
import operator

class mmqgis_attribute_export(QDialog, Ui_mmqgis_attribute_export_form):
	def __init__(self, iface):
		QDialog.__init__(self)
		self.iface = iface
		self.setupUi(self)
		QObject.connect(self.browseoutfile, SIGNAL("clicked()"), self.browse_outfiles)
        	QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("accepted()"), self.run)

		for name, layer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
			if layer.type() == QgsMapLayer.VectorLayer:
				self.sourcelayer.addItem(layer.name())

		QObject.connect(self.sourcelayer, SIGNAL("currentIndexChanged(QString)"), self.set_attributes)
		self.set_attributes()

		self.outfilename.setText(os.getcwd() + "/attributes.csv")

        def browse_outfiles(self):
		newname = QFileDialog.getSaveFileName(None, QString.fromLocal8Bit("Output CSV File"), 
			self.outfilename.displayText(), "*.csv")
                if newname != None:
                	self.outfilename.setText(QString(newname))

	def set_attributes(self):
		self.attributes.clear()
		layername = self.sourcelayer.currentText()
		for name, searchlayer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
			if searchlayer.name() == layername:
				for index, field in searchlayer.dataProvider().fields().iteritems():
					self.attributes.addItem(field.name())

	def run(self):
		layer = None
		layername = str(self.sourcelayer.currentText())
		for name, searchlayer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
			if searchlayer.name() == layername:
				layer = searchlayer

		if layer == None:
			QMessageBox.information(self.iface.mainWindow(), "Attribute Export", \
				"Project has no active vector layer to join")
			return

		if len(self.outfilename.displayText()) <= 0:
			QMessageBox.information(self.iface.mainWindow(), "Attribute Export", "No output CSV file given")
			return

		# Compile a list and header of selected attributes
		header = []
		attribute_list = []
		index = 0
		while self.attributes.item(index):
			list_item = self.attributes.item(index)
			if list_item.isSelected():
				attribute_list.append(index)
				header.append(str(list_item.text()))
			index += 1

		if (len(header) <= 0):
			QMessageBox.information(self.iface.mainWindow(), \
				"Attribute Export", "No attributes selected for export")
			return

		# Create the CSV file
		try:
			outfile = open(self.outfilename.displayText(), 'w')
    		except:
			QMessageBox.information(self.iface.mainWindow(), "Attribute Export", \
				"Failure opening " + self.outfilename.displayText())
			return

		writer = csv.writer(outfile)
		writer.writerow(header)


		# Iterate through each feature in the source layer
		output_count = 0
		feature_count = layer.dataProvider().featureCount()

		feature = QgsFeature()
		layer.dataProvider().select(layer.dataProvider().attributeIndexes())
		layer.dataProvider().rewind()
                while layer.dataProvider().nextFeature(feature):
			self.iface.mainWindow().statusBar().showMessage \
				("Exporting feature " + str(feature.id()) + " of " + str(feature_count))
			attributes = feature.attributeMap()

			row = []
			for column in attribute_list:
				row.append(attributes[column].toString())

			writer.writerow(row)

		del writer

		self.iface.mainWindow().statusBar().showMessage(str(feature_count) + " records exported")
