# --------------------------------------------------------
#    mmqgis_attribute_join - Sort shapefile by attribute
#
#    begin                : August 9, 2009
#    copyright            : (c) 2009 by Michael Minn
#    email                : See michaelminn.com
#
#   MMQGIS is free software and is offered without guarantee
#   or warranty. You can redistribute it and/or modify it 
#   under the terms of version 2 of the GNU General Public 
#   License (GPL v2) as published by the Free Software 
#   Foundation (www.gnu.org).
# --------------------------------------------------------

from PyQt4.QtCore import *
from PyQt4.QtGui import *
from qgis.core import *
from mmqgis_attribute_join_form import *

import csv
import os.path
import operator

class mmqgis_attribute_join(QDialog, Ui_mmqgis_attribute_join_form):
	def __init__(self, iface):
		QDialog.__init__(self)
		self.iface = iface
		self.setupUi(self)
		QObject.connect(self.browse_infile, SIGNAL("clicked()"), self.browse_infiles)
		QObject.connect(self.browse_outfile, SIGNAL("clicked()"), self.browse_outfiles)
		QObject.connect(self.browse_notfound, SIGNAL("clicked()"), self.browse_notfoundfiles)
        	QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("accepted()"), self.run)

		for name, layer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
			if layer.type() == QgsMapLayer.VectorLayer:
				self.joinlayer.addItem(layer.name())

		QObject.connect(self.joinlayer, SIGNAL("currentIndexChanged(QString)"), self.set_join_attributes)
		self.set_join_attributes()

		self.outfilename.setText(os.getcwd() + "/joined.shp")
		self.notfoundfilename.setText(os.getcwd() + "/notfound.csv")

        def browse_infiles(self):
		newname = QFileDialog.getOpenFileName(None, QString.fromLocal8Bit("Input CSV File"), 
			self.infilename.displayText(), "*.csv")

                if newname:
			try:
				infile = open(newname, 'r')
			except:
				QMessageBox.information(self.iface.mainWindow(), "Input CSV File", "Failure opening " + newname)
			else:
				dialect = csv.Sniffer().sniff(infile.read(1024))
				infile.seek(0)
				reader = csv.reader(infile, dialect)
				header = reader.next()
				del reader
				del infile

				self.csvfilefield.clear()
				for field in header:
					self.csvfilefield.addItem(QString(field))

                		self.infilename.setText(QString(newname))

        def browse_outfiles(self):
		newname = QFileDialog.getSaveFileName(None, QString.fromLocal8Bit("Output Shapefile"), 
			self.outfilename.displayText(), "*.shp")
                if newname != None:
                	self.outfilename.setText(QString(newname))

        def browse_notfoundfiles(self):
		newname = QFileDialog.getSaveFileName(None, QString.fromLocal8Bit("Output Unjoined CSV File"), 
			self.notfoundfilename.displayText(), "*.csv")
                if newname != None:
                	self.notfoundfilename.setText(QString(newname))

	def set_join_attributes(self):
		self.joinlayerattribute.clear()
		layername = self.joinlayer.currentText()
		for name, searchlayer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
			if searchlayer.name() == layername:
				for index, field in searchlayer.dataProvider().fields().iteritems():
					self.joinlayerattribute.addItem(field.name())

	def run(self):
		layer = None
		layername = str(self.joinlayer.currentText())
		for name, searchlayer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
			if searchlayer.name() == layername:
				layer = searchlayer

		if layer == None:
			QMessageBox.information(self.iface.mainWindow(), "Attribute Join", \
				"Project has no active vector layer to join")
			return

		if len(self.infilename.displayText()) <= 0:
			QMessageBox.information(self.iface.mainWindow(), "Attribute Join", "No input CSV file given")
			return

		# Create a combined field list from the source layer and the CSV file header
		try:
			infile = open(self.infilename.displayText(), 'r')
		except:
			QMessageBox.information(self.iface.mainWindow(), "Attribute Join", 
				"Failure opening " + self.infilename.displayText())
			return
			
		dialect = csv.Sniffer().sniff(infile.read(1024))
		infile.seek(0)
		reader = csv.reader(infile, dialect)

                fields = layer.dataProvider().fields()
		layer_field_count = len(fields)

		header = reader.next()
		for field in header:
			fields[len(fields)] = QgsField(field[0:9].strip(), QVariant.String)

		# Create the output shapefile
		outfilename = self.outfilename.displayText()
		if QFile(outfilename).exists():
			if not QgsVectorFileWriter.deleteShapeFile(QString(outfilename)):
				QMessageBox.information(self.iface.mainWindow(), "Attribute Join", 
					"Failure deleting existing shapefile: " + savename)
				return

                outfile = QgsVectorFileWriter(QString(outfilename), QString("System"), \
				fields, layer.dataProvider().geometryType(), layer.dataProvider().crs())
		if (outfile.hasError() != QgsVectorFileWriter.NoError):
			QMessageBox.information(self.iface.mainWindow(), "Attribute Join", 
				"Failure creating output shapefile:" + str(outfile.hasError()));

		# Read the CSV file data into memory
		csv_data = []
		csv_found = []
		for row in reader:
			csv_data.append(row)
			csv_found.append(0)

		joinfield_index = self.csvfilefield.currentIndex();
		for x in range(0, len(csv_data)):
			csv_data[x][joinfield_index] = csv_data[x][joinfield_index].strip().lower()

		del reader

		# Iterate through each feature in the source layer
		joinlayer_attribute = self.joinlayerattribute.currentIndex();
		matched_count = 0
		feature_count = layer.dataProvider().featureCount()

		#feature_id = 1
		#while layer.featureAtId(feature_id, feature):
		#	feature_id += 1

		feature = QgsFeature()
		layer.dataProvider().select(layer.dataProvider().attributeIndexes())
		layer.dataProvider().rewind()
                while layer.dataProvider().nextFeature(feature):
			self.iface.mainWindow().statusBar().showMessage("Joining feature " + str(feature.id()) + \
				" of " + str(feature_count) + " (" + str(matched_count) + " matched)")
			attributes = feature.attributeMap()
			key = str(attributes[joinlayer_attribute].toString()).lower()

			row_index = 0
			for row in csv_data:
				if row[joinfield_index] == key:
					newattributes = {}
					for name, value in attributes.iteritems():
						newattributes[len(newattributes)] = value
					
					for combine in row:
						newattributes[len(newattributes)] = QVariant(combine)
			
					newfeature = QgsFeature()
					newfeature.setAttributeMap(newattributes)
					newfeature.setGeometry(feature.geometry())
					outfile.addFeature(newfeature)
					matched_count += 1
					csv_found[row_index] += 1
				row_index += 1

		del outfile
		del infile

		# Write records that were not joined to the notfound file
		try:
			outfile = open(self.notfoundfilename.displayText(), 'w')
		except:
			QMessageBox.information(self.iface.mainWindow(), "Attribute Join", 
				"Failure opening " + self.notfoundfilename.displayText())
		else:
			writer = csv.writer(outfile, dialect)
			writer.writerow(header)
			for x in range(0, len(csv_data)):
				if not csv_found[x]:
					writer.writerow(csv_data[x])
			del writer
			del outfile
		
		if self.addtoproject.isChecked():
			vlayer = self.iface.addVectorLayer(outfilename, "joined", "ogr")

			
		self.iface.mainWindow().statusBar().showMessage(str(matched_count) + " records joined from " + \
			str(feature_count) + " shape records and " + str(len(csv_data)) + " CSV file records")
