# --------------------------------------------------------
#    mmqgis_gridify - Snap shape verticies to grid
#
#    begin                : August 5, 2009
#    copyright            : (c) 2009 by Michael Minn
#    email                : See michaelminn.com
#
#   MMQGIS is free software and is offered without guarantee
#   or warranty. You can redistribute it and/or modify it 
#   under the terms of version 2 of the GNU General Public 
#   License (GPL v2) as published by the Free Software 
#   Foundation (www.gnu.org).
# --------------------------------------------------------

import os.path
from PyQt4.QtCore import *
from PyQt4.QtGui import *
from qgis.core import *
from mmqgis_gridify_form import *


class mmqgis_gridify(QDialog, Ui_mmqgis_gridify_form):
	def __init__(self, iface):
		QDialog.__init__(self)
		self.iface = iface
		self.setupUi(self)
		QObject.connect(self.browse, SIGNAL("clicked()"), self.browse_outfile)
        	QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("accepted()"), self.run)

		for name, layer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
			if layer.type() == QgsMapLayer.VectorLayer:
				self.sourcelayer.addItem(layer.name())

		QObject.connect(self.sourcelayer, SIGNAL("currentIndexChanged(int)"), self.layer_changed)
		
		self.filename.setText(os.getcwd() + "/gridified.shp")
		

        def browse_outfile(self):
		newname = QFileDialog.getSaveFileName(None, QString.fromLocal8Bit("Output Shapefile"), 
			self.filename.displayText(), "*.shp")
                if newname != None:
                	self.filename.setText(QString(newname))

	def layer_changed(self):
		layer = None
		layername = str(self.sourcelayer.currentText())
		for name, searchlayer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
			if searchlayer.name() == layername:
				layer = searchlayer

		extent = QgsRectangle (-10.0, -10.0, 10.0, 20.0)
		if (layer):
			extent = layer.extent()

		#if (self.iface.mapCanvas() != None) and (self.iface.mapCanvas().mapRenderer() != None):
		#	extent = self.iface.mapCanvas().mapRenderer().extent()
		#elif self.iface.activeLayer():
		#	extent = self.iface.activeLayer().extent()

		self.hspacing.setText(str(extent.width() / 200))
		self.vspacing.setText(str(extent.height() / 200))

	def run(self):
		layer = None
		layername = str(self.sourcelayer.currentText())
		for name, searchlayer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
			if searchlayer.name() == layername:
				layer = searchlayer

		if not layer:
			QMessageBox.information(self.iface.mainWindow(), "Gridify Layer", \
				"Project has no active vector layer to gridify")
			return

		savename = str(self.filename.displayText()).strip()
		if savename == "":
			QMessageBox.information(self.iface.mainWindow(), "Grid", "No output filename given")
			return

		hspacing = float(self.hspacing.displayText())
		vspacing = float(self.hspacing.displayText())
		if (hspacing <= 0) or (vspacing <= 0):
			QMessageBox.information(self.iface.mainWindow(), "Grid", 
				"Invalid grid spacing: " + str(hspacing) + "/" + str(vspacing))
			return

		if QFile(savename).exists():
			if not QgsVectorFileWriter.deleteShapeFile(QString(savename)):
				QMessageBox.information(self.iface.mainWindow(), "Grid", 
					"Failure deleting existing shapefile: " + savename)
				return

                outfile = QgsVectorFileWriter(QString(savename), QString("System"), layer.dataProvider().fields(),
				layer.dataProvider().geometryType(), layer.dataProvider().crs())

		if (outfile.hasError() != QgsVectorFileWriter.NoError):
			QMessageBox.information(self.iface.mainWindow(), "Grid", 
				"Failure creating output shapefile:" + str(outfile.hasError()));
			return

		self.point_count = 0
		self.deleted_points = 0

		feature_number = 0
		feature_count = layer.dataProvider().featureCount()
                layer.dataProvider().select(layer.dataProvider().attributeIndexes())
		layer.dataProvider().rewind()

		feature = QgsFeature()
                while layer.dataProvider().nextFeature(feature):
			self.iface.mainWindow().statusBar().showMessage("Gridifying feature " + str(feature_number + 1))

			geometry = feature.geometry()

			if geometry.wkbType() == QGis.WKBPoint:
				points = self.gridify_points(hspacing, vspacing, [geometry.asPoint()])
				geometry = geometry.fromPoint(points[0])

			elif geometry.wkbType() == QGis.WKBLineString:
				#print "LineString"
				polyline = self.gridify_points(hspacing, vspacing, geometry.asPolyline())
				if len(polyline) < 2:
					geometry = None
				else:
					geometry = geometry.fromPolyline(polyline)

			elif geometry.wkbType() == QGis.WKBPolygon:
				newpolygon = []
				for polyline in geometry.asPolygon():
					newpolyline = self.gridify_points(hspacing, vspacing, polyline)
					if len(newpolyline) > 1:
						newpolygon.append(newpolyline)

				if len(newpolygon) <= 0:
					geometry = None
				else:
					geometry = geometry.fromPolygon(newpolygon)

			elif geometry.wkbType() == QGis.WKBMultiPoint:
				multipoints = self.gridify_points(hspacing, vspacing, geometry.asMultiPoint())
				geometry = geometry.fromMultiPoint(multipoints)

			elif geometry.wkbType() == QGis.WKBMultiLineString:
				#print "MultiLineString"
				newmultipolyline = []
				for polyline in geometry.asMultiPolyline():
					newpolyline = self.gridify_points(hspacing, vspacing, polyline)
					if len(newpolyline) > 1:
						newmultipolyline.append(newpolyline)

				if len(newmultipolyline) <= 0:
					geometry = None
				else:
					geometry = geometry.fromMultiPolyline(newmultipolyline)

			elif geometry.wkbType() == QGis.WKBMultiPolygon:
				#print "MultiPolygon"
				newmultipolygon = []
				for polygon in geometry.asMultiPolygon():
					newpolygon = []
					for polyline in polygon:
						newpolyline = self.gridify_points(hspacing, vspacing, polyline)
						if len(newpolyline) > 2:
							newpolygon.append(newpolyline)

					if len(newpolygon) > 0:
						newmultipolygon.append(newpolygon)

				if len(newmultipolygon) <= 0:
					geometry = None
				else:
					geometry = geometry.fromMultiPolygon(newmultipolygon)

			else:
				print "Unknown geometry type " + str(geometry.wkbType()) + \
					" on feature " + str(feature_number + 1)

			# print "Closing feature"
	
			if geometry != None:
				out_feature = QgsFeature()
				out_feature.setGeometry(geometry)
				out_feature.setAttributeMap(feature.attributeMap())
				outfile.addFeature(out_feature)

			feature_number += 1

		del outfile

		if self.addtoproject.isChecked():
			vlayer = self.iface.addVectorLayer(savename, "gridified", "ogr")
			
		self.iface.mainWindow().statusBar().showMessage("Gridified shapefile created (" + \
			str(self.deleted_points) + " of " + str(self.point_count) + " points deleted)")


	def gridify_points(self, hspacing, vspacing, points):
		# Align points to grid
		newpoints = []
		for point in points:
			self.point_count += 1
			newpoints.append(QgsPoint(round(point.x() / hspacing, 0) * hspacing, \
					    round(point.y() / vspacing, 0) * vspacing))

		# Delete overlapping points
		z = 0
		while z < (len(newpoints) - 2):
			if newpoints[z] == newpoints[z + 1]:
				newpoints.pop(z + 1)
				self.deleted_points += 1
			else:
				z += 1

		# Delete line points that go out and return to the same place
		z = 0
		while z < (len(newpoints) - 3):
			if newpoints[z] == newpoints[z + 2]:
				newpoints.pop(z + 1)
				newpoints.pop(z + 1)
				self.deleted_points += 2
				# Step back to catch arcs
				if (z > 0):
					z -= 1
			else:
				z += 1

		# Delete overlapping start/end points
		while (len(newpoints) > 1) and (newpoints[0] == newpoints[len(newpoints) - 1]):
			newpoints.pop(len(newpoints) - 1)
			self.deleted_points += 2
				
		return newpoints



