# --------------------------------------------------------
#    mmqgis_label - Create single label points for
#		    single- or multi-feature items
#
#    begin                : August 26, 2009
#    copyright            : (c) 2009 by Michael Minn
#    email                : See michaelminn.com
#
#   MMQGIS is free software and is offered without guarantee
#   or warranty. You can redistribute it and/or modify it 
#   under the terms of version 2 of the GNU General Public 
#   License (GPL v2) as published by the Free Software 
#   Foundation (www.gnu.org).
# --------------------------------------------------------

from PyQt4.QtCore import *
from PyQt4.QtGui import *
from qgis.core import *
from mmqgis_label_form import *

import os.path
import operator

class mmqgis_label_point():
	def __init__(self, name, attributemap):
		self.name = name
		self.xsum = 0
		self.ysum = 0
		self.feature_count = 0
		self.attributes = attributemap

class mmqgis_label(QDialog, Ui_mmqgis_label_form):
	def __init__(self, iface):
		QDialog.__init__(self)
		self.iface = iface
		self.setupUi(self)
		QObject.connect(self.browse, SIGNAL("clicked()"), self.browse_outfile)
        	QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("accepted()"), self.run)

		for name, layer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
			if layer.type() == QgsMapLayer.VectorLayer:
				self.sourcelayer.addItem(layer.name())

		QObject.connect(self.sourcelayer, SIGNAL("currentIndexChanged(QString)"), self.set_label_attributes)

		self.set_label_attributes()

		self.filename.setText(os.getcwd() + "/label.shp")

        def browse_outfile(self):
		newname = QFileDialog.getSaveFileName(None, QString.fromLocal8Bit("Output Shapefile"), 
			self.filename.displayText(), "*.shp")
                if newname != None:
                	self.filename.setText(QString(newname))

	def set_label_attributes(self):
		self.labelattribute.clear()
		layername = self.sourcelayer.currentText()
		for name, searchlayer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
			if searchlayer.name() == layername:
				for index, field in searchlayer.dataProvider().fields().iteritems():
					self.labelattribute.addItem(field.name())

	def run(self):
		layer = None
		layername = str(self.sourcelayer.currentText())
		for name, searchlayer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
			if searchlayer.name() == layername:
				layer = searchlayer

		if layer == None:
			QMessageBox.information(self.iface.mainWindow(), "Label Layer", "Invalid layer name " . layername)
			return

		labelattributename = self.labelattribute.currentText()
		labelindex = layer.dataProvider().fieldNameIndex(labelattributename)
		if labelindex < 0:
			QMessageBox.information(self.iface.mainWindow(), "Label Layer", \
				"Invalid label field name: " + labelattributename)
		print  "labelindex = " + str(labelindex)

		savename = str(self.filename.displayText()).strip()
		if savename == "":
			QMessageBox.information(self.iface.mainWindow(), "Label Layer", "No output filename given")
			return

		# Open file (delete any existing)
		if QFile(savename).exists():
			if not QgsVectorFileWriter.deleteShapeFile(QString(savename)):
				QMessageBox.information(self.iface.mainWindow(), "Label Layer", 
					"Failure deleting existing shapefile: " + savename)
				return

                outfile = QgsVectorFileWriter(QString(savename), QString("System"), layer.dataProvider().fields(),
				QGis.WKBPoint, layer.dataProvider().crs())

		if (outfile.hasError() != QgsVectorFileWriter.NoError):
			QMessageBox.information(self.iface.mainWindow(), "Label", 
				"Failure creating output shapefile:" + str(outfile.hasError()));
			return

		# Build dictionary of items, averaging center for multi-feature items
		features = {}
		feature = QgsFeature()
		readcount = 0
		feature_count = layer.featureCount()
		layer.dataProvider().select(layer.dataProvider().attributeIndexes())
		layer.dataProvider().rewind()
                while layer.dataProvider().nextFeature(feature):
			key = str(feature.attributeMap()[labelindex].toString())
			if not features.has_key(key):
				features[key] = mmqgis_label_point(key, feature.attributeMap())

			center = feature.geometry().boundingBox().center();
			features[key].xsum += center.x()
			features[key].ysum += center.y()
			features[key].feature_count += 1
	
			readcount += 1
			if not (readcount % 10):
				self.iface.mainWindow().statusBar().showMessage( \
					"Reading feature " + str(readcount) + " of " + str(feature_count))

		# Sort keys so features appear in alphabetical order
		keys = features.keys()
		keys.sort()

		# Calculate points and write them to the output file
		writecount = 0
		for key in keys:
			label_point = features[key]
			point = QgsPoint(label_point.xsum / label_point.feature_count, 
				label_point.ysum / label_point.feature_count)

			feature = QgsFeature()
			geometry = QgsGeometry()
			feature.setGeometry(geometry.fromPoint(point))
			feature.setAttributeMap(label_point.attributes)

			if not outfile.addFeature(feature):
				QMessageBox.information(self.iface.mainWindow(), "Label Layer", \
					"Failure writing feature to shapefile")
				break

			writecount += 1
			if not (writecount % 10):
				self.iface.mainWindow().statusBar().showMessage( \
					"Writing feature " + str(writecount) + " of " + str(len(features)))

		del outfile

		layer = self.iface.addVectorLayer(savename, "labels", "ogr")
		# layer.enableLabels()
			
		self.iface.mainWindow().statusBar().showMessage(str(writecount) + " label shapefile created from " + layername)

