# --------------------------------------------------------
#    mmqgis_merge - Merge layers to single shapefile
#
#    begin                : September 1, 2009
#    copyright            : (c) 2009 by Michael Minn
#    email                : See michaelminn.com
#
#   MMQGIS is free software and is offered without guarantee
#   or warranty. You can redistribute it and/or modify it 
#   under the terms of version 2 of the GNU General Public 
#   License (GPL v2) as published by the Free Software 
#   Foundation (www.gnu.org).
# --------------------------------------------------------

from PyQt4.QtCore import *
from PyQt4.QtGui import *
from qgis.core import *
from mmqgis_merge_form import *

import csv
import os.path
import operator

class mmqgis_merge(QDialog, Ui_mmqgis_merge_form):
	def __init__(self, iface):
		QDialog.__init__(self)
		self.iface = iface
		self.setupUi(self)
		QObject.connect(self.browseoutfile, SIGNAL("clicked()"), self.browse_outfiles)
        	QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("accepted()"), self.run)

		self.sourcelayers.clear()
		for name, layer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
			if layer.type() == QgsMapLayer.VectorLayer:
				self.sourcelayers.addItem(QString(layer.name()))
				self.sourcelayers.item(self.sourcelayers.count() - 1).setSelected(1)

		self.outfilename.setText(os.getcwd() + "/merged.shp")

        def browse_outfiles(self):
		newname = QFileDialog.getSaveFileName(None, QString.fromLocal8Bit("Output Shapefile"), 
			self.outfilename.displayText(), "*.shp")
                if newname != None:
                	self.outfilename.setText(QString(newname))

	def run(self):
		fields = {}
		layers = []
		crs = None
		geometry = None
		totalfeaturecount = 0

		index = 0
		while self.sourcelayers.item(index):
			list_item= self.sourcelayers.item(index)
			index += 1
			if list_item.isSelected():
				layername = str(list_item.text())

				# Find the selected layer by name and add to the list of layers
				layer = None
				for name, searchlayer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
					if searchlayer.name() == layername:
						layer = searchlayer

				if not layer:
					QMessageBox.information(self.iface.mainWindow(), 
						"Merge Layers", "Layer " + layername + " not found")
					return

				# Verify that all layers are the same type (point, polygon, etc)
				if (len(layers) <= 0):
					crs = layer.dataProvider().crs()
					geometry = layer.dataProvider().geometryType()

				elif (geometry != layer.dataProvider().geometryType()):
					QMessageBox.information(self.iface.mainWindow(), 
						"Merge Layers", "Merged layers must all be same type of geometry")
					return
					
				layers.append(layer)
				totalfeaturecount += layer.featureCount()

				# Add any fields not in the composite field list
				for sindex, sfield in layer.dataProvider().fields().iteritems():
					found = None
					for dindex, dfield in fields.iteritems():
						if (dfield.name() == sfield.name()) and (dfield.type() == sfield.type()):
							found = dfield

					if not found:
						fields[len(fields)] = sfield
				
		if (len(layers) <= 0):
			QMessageBox.information(self.iface.mainWindow(), "Merge Layers", "No layers given to merge")
			return
		
		# Create the output shapefile
		savename = str(self.outfilename.displayText()).strip()
		if savename == "":
			QMessageBox.information(self.iface.mainWindow(), "Merge Layers", "No output filename given")
			return

		if QFile(savename).exists():
			if not QgsVectorFileWriter.deleteShapeFile(QString(savename)):
				QMessageBox.information(self.iface.mainWindow(), "Merge Layers", 
					"Failure deleting existing shapefile: " + savename)
				return

                outfile = QgsVectorFileWriter(QString(savename), QString("System"), fields, geometry, crs)

		if (outfile.hasError() != QgsVectorFileWriter.NoError):
			QMessageBox.information(self.iface.mainWindow(), "Merge Layers", 
				"Failure creating output shapefile:" + str(outfile.hasError()));
			return

		# Copy layer features to output file
		featurecount = 0
		for layer in layers:
			feature = QgsFeature()
			layer.dataProvider().select(layer.dataProvider().attributeIndexes())
			layer.dataProvider().rewind()
	                while layer.dataProvider().nextFeature(feature):
				sattributes = feature.attributeMap()
				dattributes = {}
				for dindex, dfield in fields.iteritems():
					dattributes[dindex] = QVariant(dfield.type())
					for sindex, sfield in layer.dataProvider().fields().iteritems():
						if (sfield.name() == dfield.name()) and (sfield.type() == dfield.type()):
							dattributes[dindex] = sattributes[sindex]
							break

				#for dindex, dfield in dattributes.iteritems():
				#	print layer.name() + " (" + str(dindex) + ") " + str(dfield.toString())

				feature.setAttributeMap(dattributes)
				outfile.addFeature(feature)
				featurecount += 1
				self.iface.mainWindow().statusBar().showMessage("Writing feature " + \
					str(featurecount) + " of " + str(totalfeaturecount))

		del outfile

		# Add the merged layer to the project
		self.iface.addVectorLayer(savename, "merged", "ogr")

		self.iface.mainWindow().statusBar().showMessage(str(featurecount) + " records exported")
