# --------------------------------------------------------
#    mmqgis_voronoi - Voronoi diagram creation
#
#    begin                : August 11, 2009
#    copyright            : (c) 2009 by Michael Minn
#    email                : See michaelminn.com
#
#   MMQGIS is free software and is offered without guarantee
#   or warranty. You can redistribute it and/or modify it 
#   under the terms of version 2 of the GNU General Public 
#   License (GPL v2) as published by the Free Software 
#   Foundation (www.gnu.org).
# --------------------------------------------------------

import os.path
import time
from math import *

from PyQt4.QtCore import *
from PyQt4.QtGui import *
from qgis.core import *
from mmqgis_voronoi_form import *

class mmqgis_voronoi_line:
	def __init__(self, x, y):
		self.x = x
		self.y = y
		self.angle = 0
		self.distance = 0

	def list(self, title):
		print title + ", " + str(self.x) + ", " + str(self.y) + \
			", angle " + str(self.angle * 180 / pi) + ", distance " + str(self.distance)

	def angleval(self):
		return self.angle


class mmqgis_voronoi(QDialog, Ui_mmqgis_voronoi_form):
	def __init__(self, iface):
		QDialog.__init__(self)
		self.iface = iface
		self.setupUi(self)
		QObject.connect(self.browse, SIGNAL("clicked()"), self.browse_outfile)
        	QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("accepted()"), self.run)

		for name, layer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
			if layer.type() == QgsMapLayer.VectorLayer:
				self.sourcelayer.addItem(layer.name())

		self.filename.setText(os.getcwd() + "/voronoi.shp")

        def browse_outfile(self):
		newname = QFileDialog.getSaveFileName(None, QString.fromLocal8Bit("Output Shapefile"), 
			self.filename.displayText(), "*.shp")
                if newname != None:
                	self.filename.setText(QString(newname))

	def run(self):
		layer = None
		layername = str(self.sourcelayer.currentText())
		for name, searchlayer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
			if searchlayer.name() == layername:
				layer = searchlayer

		if layer == None:
			QMessageBox.information(self.iface.mainWindow(), "Voronoi Diagram", \
				"Project has no active vector layer")
			return

		savename = str(self.filename.displayText()).strip()
		if savename == "":
			QMessageBox.information(self.iface.mainWindow(), "Grid", "No output filename given")
			return

		#self.crs = QgsCoordinateReferenceSystem()
		#if self.iface.mapCanvas() and self.iface.mapCanvas().mapRenderer():
		#	self.crs = self.iface.mapCanvas().mapRenderer().destinationSrs()
		#else:
		#	self.crs.createFromProj4("+proj=longlat +ellps=WGS84 +datum=WGS84 +no_defs")

		#fields = { 0 : QgsField("CENTERX", QVariant.Double, "real", 24, 16),
		#	   1 : QgsField("CENTERY", QVariant.Double, "real", 24, 16) }


		if QFile(savename).exists():
			if not QgsVectorFileWriter.deleteShapeFile(QString(savename)):
				QMessageBox.information(self.iface.mainWindow(), "Voronoi Diagram", 
					"Failure deleting existing shapefile: " + savename)
				return

                outfile = QgsVectorFileWriter(QString(savename), QString("System"), layer.dataProvider().fields(), \
				QGis.WKBPolygon, layer.dataProvider().crs())
		if (outfile.hasError() != QgsVectorFileWriter.NoError):
			QMessageBox.information(self.iface.mainWindow(), "Voronoi Diagram", 
				"Failure creating output shapefile:" + str(outfile.hasError()));

		points = []
		xmin = 0
		xmax = 0
		ymin = 0
		ymax = 0
		feature = QgsFeature()
		layer.dataProvider().select(layer.dataProvider().attributeIndexes())
		layer.dataProvider().rewind()

                while layer.dataProvider().nextFeature(feature):
			# Re-read by feature ID because nextFeature() doesn't always seem to read attributes
			layer.featureAtId(feature.id(), feature)
			geometry = feature.geometry()
			self.iface.mainWindow().statusBar().showMessage("Reading feature " + str(feature.id()))
			if geometry.wkbType() == QGis.WKBPoint:
				points.append( (geometry.asPoint().x(), geometry.asPoint().y(), feature.attributeMap()) )
				if (len(points) <= 1) or (xmin > geometry.asPoint().x()):
					xmin = geometry.asPoint().x()
				if (len(points) <= 1) or (xmax < geometry.asPoint().x()):
					xmax = geometry.asPoint().x()
				if (len(points) <= 1) or (ymin > geometry.asPoint().y()):
					ymin = geometry.asPoint().y()
				if (len(points) <= 1) or (ymax < geometry.asPoint().y()):
					ymax = geometry.asPoint().y()

		if (len(points) < 3):
			QMessageBox.information(self.iface.mainWindow(), "Voronoi", "Too few points to create diagram")
			return

		for center in points:
		# for center in [ points[17] ]:
			# print "\nCenter, " + str(center[0]) + ", " + str(center[1])
			self.iface.mainWindow().statusBar().showMessage("Processing point " + \
				str(center[0]) + ", " + str(center[1]))

			# Borders are tangents to midpoints between all neighbors
			tangents = []
			for neighbor in points:
				border = mmqgis_voronoi_line((center[0] + neighbor[0]) / 2.0, (center[1] + neighbor[1]) / 2.0)
				if ((neighbor[0] != center[0]) or (neighbor[1] != center[1])):
					tangents.append(border)

			# Add edge intersections to clip to extent of points
			offset = (xmax - xmin) * 0.01
			tangents.append(mmqgis_voronoi_line(xmax + offset, center[1]))
			tangents.append(mmqgis_voronoi_line(center[0], ymax + offset))
			tangents.append(mmqgis_voronoi_line(xmin - offset, center[1]))
			tangents.append(mmqgis_voronoi_line(center[0], ymin - offset))
			#print "Extent x = " + str(xmax) + " -> " + str(xmin) + ", y = " + str(ymax) + " -> " + str(ymin)

			# Find vector distance and angle to border from center point
			for scan in range(0, len(tangents)):
				run = tangents[scan].x - center[0]
				rise = tangents[scan].y - center[1]
				tangents[scan].distance = sqrt((run * run) + (rise * rise))
				if (tangents[scan].distance <= 0):
					tangents[scan].angle = 0
				elif (tangents[scan].y >= center[1]):
					tangents[scan].angle = acos(run / tangents[scan].distance)
				elif (tangents[scan].y < center[1]):
					tangents[scan].angle = (2 * pi) - acos(run / tangents[scan].distance)
				elif (tangents[scan].x > center[0]):
					tangents[scan].angle = pi / 2.0
				else:
					tangents[scan].angle = 3 * pi / 4

				#print "  Tangent, " + str(tangents[scan].x) + ", " + str(tangents[scan].y) + \
				#	", angle " + str(tangents[scan].angle * 180 / pi) + ", distance " + \
				#	str(tangents[scan].distance)


			# Find the closest line - guaranteed to be a border
			closest = -1
			for scan in range(0, len(tangents)):
				if ((closest == -1) or (tangents[scan].distance < tangents[closest].distance)):
					closest = scan

			# Use closest as the first border
			border = mmqgis_voronoi_line(tangents[closest].x, tangents[closest].y)
			border.angle = tangents[closest].angle
			border.distance = tangents[closest].distance
			borders = [ border ]

			#print "  Border 0) " + str(closest) + " of " + str(len(tangents)) + ", " \
			#	+ str(border.x) + ", " + str(border.y) \
			#	+ ", (angle " + str(border.angle * 180 / pi) + ", distance " \
			#	+ str(border.distance) + ")"

			# Work around the tangents in a CCW circle
			circling = 1
			while circling:
				next = -1
				scan = 0
				while (scan < len(tangents)):
					anglebetween = tangents[scan].angle - borders[len(borders) - 1].angle
					if (anglebetween < 0):
						anglebetween += (2 * pi)
					elif (anglebetween > (2 * pi)):
						anglebetween -= (2 * pi)

					#print "    Scanning " + str(scan) + " of " + str(len(borders)) + \
					#	", " + str(tangents[scan].x) + ", " + str(tangents[scan].y) + \
					#	", angle " + str(tangents[scan].angle * 180 / pi) + \
					#	", anglebetween " + str(anglebetween * 180 / pi)

					# If border intersects to the left
					if (anglebetween < pi) and (anglebetween > 0):
						# A typo here with a reversed slash cost 8/13/2009 debugging
						tangents[scan].iangle = atan2( (tangents[scan].distance / 
							borders[len(borders) - 1].distance) \
							- cos(anglebetween), sin(anglebetween))
						tangents[scan].idistance = borders[len(borders) - 1].distance \
							/ cos(tangents[scan].iangle)

						tangents[scan].iangle += borders[len(borders) - 1].angle

						# If the rightmost intersection so far, it's a candidate for next border
						if (next < 0) or (tangents[scan].iangle < tangents[next].iangle):
							# print "      Take idistance " + str(tangents[scan].idistance)
							next = scan

					scan += 1

				# iangle/distance are for intersection of border with next border
				borders[len(borders) - 1].iangle = tangents[next].iangle
				borders[len(borders) - 1].idistance = tangents[next].idistance

				# Stop circling if back to the beginning
				if (borders[0].x == tangents[next].x) and (borders[0].y == tangents[next].y):
					circling = 0

				else:
					# Add the next border
					border = mmqgis_voronoi_line(tangents[next].x, tangents[next].y)
					border.angle = tangents[next].angle
					border.distance = tangents[next].distance
					border.iangle = tangents[next].iangle
					border.idistance = tangents[next].idistance
					borders.append(border)
					#print "  Border " + str(len(borders) - 1) + \
					#	") " + str(next) + ", " + str(border.x) + \
					#	", " + str(border.y) + ", angle " + str(border.angle * 180 / pi) +\
					#	", iangle " + str(border.iangle * 180 / pi) +\
					#	", idistance " + str(border.idistance) + "\n"

				# Remove the border from the list so not repeated
				tangents.pop(next)
				if (len(tangents) <= 0):
					circling = 0

			if len(borders) >= 3:
				polygon = []
				for border in borders:
					ix = center[0] + (border.idistance * cos(border.iangle))
					iy = center[1] + (border.idistance * sin(border.iangle))
					#print "  Node, " + str(ix) + ", " + str(iy) + \
					#	", angle " + str(border.angle * 180 / pi) + \
					#	", iangle " + str(border.iangle * 180 / pi) + \
					#	", idistance " + str(border.idistance) + ", from " \
					#	+ str(border.x) + ", " + str(border.y)
					polygon.append(QgsPoint(ix, iy))

				# attributes = { 0:QVariant(center[0]), 1:QVariant(center[1]) }

				geometry = QgsGeometry.fromPolygon([ polygon ])
				feature = QgsFeature()
				feature.setGeometry(geometry)
				feature.setAttributeMap(center[2])
				outfile.addFeature(feature)
				
				
                del outfile

		if self.addtoproject.isChecked():
			vlayer = self.iface.addVectorLayer(savename, "voronoi", "ogr")

		self.iface.mainWindow().statusBar().showMessage("Created " + str(len(points)) + " polygon Voronoi diagram")
